/*
Copyright 2016 OpenMarket Ltd

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/
"use strict";

var Olm = require("olm");
//var q = require("q");
var MatrixEvent = require("../models/event").MatrixEvent;


var OLM_GROUP_ALGORITHM = "m.olm.v1.megolm-sha2";

/* remember to .free() the result of this */
function _getAccount(client) {
    var account = new Olm.Account();
    var e2eAccount = client.sessionStore.getEndToEndAccount();
    account.unpickle(client.pickleKey, e2eAccount);
    return account;
}

/**
 * @constructor
 * @implements {CryptoClient}
 *
 * @param {Object} opts The configuration options for this client.
 * @param {MatrixHttpApi} opts.httpApi
 * @param {object} opts.sessionStore
 * @param {string} opts.deviceId
 * @param {string} opts.userId The user ID for this user.
 */
function OlmGroupCryptoClient(opts) {
    this.sessionStore = opts.sessionStore;
    this.deviceId = opts.deviceId;
    this._http = opts.httpApi;

    this.pickleKey = "DEFAULT_KEY";

    var account = _getAccount(this);
    try {
        var e2eKeys = JSON.parse(account.identity_keys());
        this.deviceCurve25519Key = e2eKeys.curve25519;
    } finally {
        account.free();
    }
}

Object.defineProperty(OlmGroupCryptoClient.prototype, "algorithm", {
    get: function() { return OLM_GROUP_ALGORITHM; },
});


/** @inheritDoc */
OlmGroupCryptoClient.prototype.startClient = function() {
};


/** @inheritDoc */
OlmGroupCryptoClient.prototype.enableRoomEncryption = function(roomId, config) {
};


/** @inheritDoc */
OlmGroupCryptoClient.prototype.encryptEvent = function(
    roomId, e2eRoomInfo, eventType, content, txnId
) {
    var ciphertext = "";
    return {
        algorithm: OLM_GROUP_ALGORITHM,
        sender_key: this.deviceCurve25519Key,
        ciphertext: ciphertext,
    };
};

/** @inheritDoc */
OlmGroupCryptoClient.prototype.decryptEvent = function(event) {
    var content = event.getContent();
    var payload = {
        roomId: event.getRoomId(),
        type: "m.room.message",
        content: content,
    };

    return new MatrixEvent({
        // TODO: Add a key to indicate that the event was encrypted.
        // TODO: Check the sender user id matches the sender key.
        origin_server_ts: event.getTs(),
        room_id: payload.roomId,
        user_id: event.getSender(),
        event_id: event.getId(),
        unsigned: event.getUnsigned(),
        type: payload.type,
        content: payload.content
    }, "encrypted");
};

/**
 * @implements CryptoAlgorithm
 */
module.exports = {
    /** @inheritDoc */
    createClient: function(opts) {
        return new OlmGroupCryptoClient(opts);
    },
};
